%% Basis Pursuit Denoising with Forward-Backward
% Test the use of Forward-backward-like splitting for the resolution
% of a compressed sensing regularization.

addpath('../');
addpath('../toolbox/');
clear;
%%
% Dimension of the problem.

n = 50;% number of vectors
p = 3000;% length of vector

%%
% Regularization parameter.

lambda = 1e-4;
lambda_e=1;
%%
% Matrix and observation.
A0 =randn(p,n);
x0=zeros(n,1);
perm = randperm(n) ;
k=ceil(n/10);
x0(perm(1:k)) = (rand(k,1)-0.5) ; % signal to be estimated
x0=x0/norm(x0);
use_error=1;
if (use_error==0)
    A=A0;
    y=A*x0;
    xpe=x0;
else
    e0=zeros(p,1);
    k_e=ceil(p/50);
    perm_e = randperm(p) ;
    e0(perm_e(1:k_e)) = 1e1*(rand(k_e,1)-0.5) ;

    % y = randn(p,1);
    y=A0*x0+e0;
    A=[A0 eye(p)];
    xpe=[x0
        e0];
end
xpe_length=length(xpe);

%%
% List of benchmarked algorithms.

methods = {'fb', 'fista', 'nesterov'};

%%
% operator callbacks
if (use_error==0)
    F = @(x)lambda*norm(x,1);
else
    F = @(x)lambda*(norm(x(1:n),1)+lambda_e*norm(x(n+1:n+p),1));
end
G = @(x)1/2*norm(y-A*x)^2;

%%
% Proximal operator of F. 

ProxF = @(x,tau)perform_soft_thresholding(x,lambda*tau);

%%
% Gradient operator of G.
GradG = @(x)A'*(A*x-y);

%%
% Lipschitz constant.

L = norm(A)^2;

%%
% Function to record the energy.

options.report = @(x)F(x)+G(x);

%%
% Bench the algorithm

options.niter = 10000;
E = [];
% for i=1:length(methods)
    options.method = methods{2};
%     [x,e] = perform_fb(zeros(xpe_length,1), ProxF, GradG, L, options);
    [x] = perform_fb(zeros(xpe_length,1), ProxF, GradG, L, options);
%     E(:,2) = e(:);
% end
% e = min(E(:));
disp(['error=' num2str(norm(x-xpe)/norm(xpe))]);
if (use_error==1)
   x_l1=x(1:n);
   e_l1=x(n+1:n+p);
   error_x=norm(x0-x_l1)/norm(x0);
   error_e=norm(e0-e_l1)/norm(e0);
   disp(['error_x=' num2str(error_x) '    error_e=' num2str(error_e)]);
end
%%
% Display the decays of the energies.

%%
% _IMPORTANT:_ Note that the comparison with Nesterov is unfair, since each
% Nesterov iteration cost twice more. 

% sel = 1:round(options.niter/10);
% loglog(E(sel,:)-e);
% axis tight;
% legend(methods);