function SparseRefinement(inputFile,outputFile,methodIndex)
addpath L1Solvers;
addpath L1Solvers/cvx;
addpath L1Solvers/toolbox_optim;
addpath L1Solvers/toolbox_optim/toolbox;
l1Method = {'Homotopy', 'FISTA','CVX'};
global R;
global referenceShape;

alpha=50;
beta=0.17;
A=R;
[M N]=size(A);
lengthsum=N+M;
[Face inputShape status] = readoff(inputFile);
%% procrustes transform
[d1,inputAligned,transform] = procrustes(referenceShape,inputShape);
y=reshape(inputAligned',M,1);

D=[A eye(M)];
%% solve the problem;  
if (length(who('methodIndex'))<=0)
    methodIndex=1;
end
if(methodIndex==1)
    tic;
    [xEst, nIter, curTimeEst, curErrEst] = SolveHomotopy(D, y,N,alpha,beta,'maxiteration', 200) ;
    tEst = toc ; 
elseif(methodIndex==2)
    F = @(x)alpha*(norm(x(1:N),1)+beta*norm(x(N+1:N+M),1));
    G = @(x)norm(y-D*x)^2;
    ProxF = @(x,tau)perform_soft_thresholding2(x,N,alpha*tau,beta*tau);
    GradG = @(x)D'*(D*x-y);
    L = norm(D)^2;
    options.report = @(x)F(x)+G(x);
    options.niter = 200;
    options.method='fista';
    tic;
    [xEst] = perform_fb(zeros(lengthsum,1), ProxF, GradG, L, options);
    tEst = toc ;
elseif(methodIndex==3)
    tic;
    cvx_begin
          variable xEst(N)
          variable eEst(M)
          minimize(alpha*norm(xEst,1)+beta*norm(eEst,1))
          subject to
              A*xEst+eEst==y;
    cvx_end
    tEst = toc ; 
end
x_l1=xEst(1:N);
yssc=A*x_l1;
%% report time
disp([l1Method{methodIndex} '    time=' num2str(tEst)]);
%% transform the shape to origrinal position
outputShape=reshape(yssc,3,length(yssc)/3)';
c = transform.c;
T = transform.T;
b = transform.b;
outputShape=((outputShape-c)*T^-1)/b;
saveoff(Face,outputShape,outputFile);
end